@@ -5,15 +5,21 @@ NULL
 #' Calculates derivatives of all shares with respect to all non-linear parameters in a given market.
 #'
 #' @param blp_data data object created by the function \code{BLP_data},
-#' @param cm_input contraction mapping object created by the function \code{getDelta_wrap},
-#' @param market character specifying the market in which derivatives are calculated
+#' @param par_theta2 matrix with column and rownames providing a starting value for the optimization routine (see details),
+#' @param market character specifying the market in which derivatives are calculated,
+#' @param printLevel level of output information (default = 1)
 #'
 #' @return Returns a numeric matrix with derivatives.
 #' Cell in row i and col j is the derivative of share i with respect to parameter j.
 #'
+#' @details NA's in \code{par_theta2} entries indicate the exclusion from estimation, i.e. the coefficient is assumed to be zero.
+#' If only unobserved heterogeneity is used (no demographics), the column name of \code{par_theta2} must be "unobs_sd".
+#' With demographics the colnames must match the names of provided demographics (as in \code{demographic_draws}) and "unobs_sd".
+#' Row names of \code{par_theta2} must match random coefficients as specified in \code{model}. Constants must be named "(Intercept)".
+#'
 #' @examples
 #' K<-2 #number of random coefficients
-#' data <- get_BLP_dataset(nmkt = 25, nbrn = 20,
+#' data <- simulate_BLP_dataset(nmkt = 25, nbrn = 20,
 #'                         Xlin = c("price", "x1", "x2", "x3", "x4", "x5"),
 #'                         Xexo = c("x1", "x2", "x3", "x4", "x5"),
 #'                         Xrandom = paste0("x",1:K),instruments = paste0("iv",1:10),
@@ -40,57 +46,55 @@ NULL
 #'                      integration_accuracy = 40,
 #'                      integration_seed = 1)
 #'
-#' theta_guesses <- matrix(c(0.5,2), nrow=2)
-#' rownames(theta_guesses) <- c("x1","x2")
-#' colnames(theta_guesses) <- "unobs_sd"
-#'
-#' blp_est <- estimateBLP(blp_data =blp_data,
-#'                        par_theta2 = theta_guesses,
-#'                        extremumCheck = FALSE ,
-#'                        printLevel = 1 )
-#' summary(blp_est)
-#'
-#' delta_eval <- getDelta_wrap(  blp_data=blp_data,
-#'                               par_theta2 = theta_guesses,
-#'                               printLevel = 4)
+#' theta2 <- matrix(c(0.5,2), nrow=2)
+#' rownames(theta2) <- c("x1","x2")
+#' colnames(theta2) <- "unobs_sd"
 #'
 #' derivatives1 <- dstdtheta_wrap(  blp_data=blp_data,
-#'                                  cm_input = delta_eval,
+#'                                   par_theta2 = theta2,
 #'                                  market = 2)
 #'
 #' @export
-dstdtheta_wrap <- function(  blp_data,
-                             cm_input,
-                             market ){
-
+dstdtheta_wrap <- function(  blp_data, par_theta2, market, printLevel = 1 ){
   nobs <- blp_data$parameters$nobs
   K <- blp_data$parameters$K
-  ## class checks
-  if(class(blp_data) != "blp_data")
-    stop("blp_data has wrong class. Call BLP_data() first.")
-
-  if(class(cm_input) != "blp_cm")
-    stop("blp_cm has wrong class. Call getDelta_wrap() first.")
-
-  ## find numeric market_id
-  if( missing(market))
-    stop("Specify a valid market.")
-
-  if( (length(market) != 1)  )
-    stop("Only one market can be specified.")
-
   original_market_id <- blp_data$parameters$market_id_char_in
   original_product_id <- blp_data$parameters$product_id
   market <- as.character(market)
 
-  if( !any(market %in% original_market_id) )
-    stop("Market is not available in provided dataset.")
+  ## check inouts
+  if(class(blp_data) != "blp_data") stop("Input has wrong class. Call BLP_data() first.")
+  if( printLevel > 0) cat("Mean utility (delta) is used as provided in the BLP_data() function.")
+  if( missing(market)) stop("Specify a valid market.")
+  if( (length(market) != 1)  ) stop("Only one market can be specified.")
+  if( !any(market %in% original_market_id) ) stop("Market is not available in provided dataset.")
 
+  ## calculate share evaluations (all markets)
+  current_delta <- blp_data$data$delta
+  start_theta2 <- .prepare_theta2(par_theta2,
+                                  final_col_names_par = c( "unobs_sd" ,
+                                                           blp_data$parameters$demographic_names),
+                                  final_row_names_par = colnames(blp_data$data$X_rand),
+                                  K = blp_data$parameters$K,
+                                  M = blp_data$parameters$total_demogr)
+  theta2Mat<- .get.theta2.reshape(theta2.in       = start_theta2$par_theta2,
+                                  totalRC         = blp_data$parameters$K,
+                                  total.demogr.in = blp_data$parameters$total_demogr,
+                                  indices.in      = start_theta2$indices,
+                                  fill            = 0 ) # NA are replaced by zeros to simplify x * par in getExpMu
+  expmu <-  getExpMu( theta2Matrix = theta2Mat,
+                      qv = blp_data$integration$drawsRcMktShape,
+                      Xrandom = blp_data$data$X_rand,
+                      cdid = blp_data$parameters$market_id,
+                      demographics = blp_data$integration$drawsDemMktShape)
+  sij <-  getSij(expmu = expmu,
+                 expdelta = exp(current_delta),
+                 cdindex = blp_data$parameters$cdindex )
+
+  ## extract market information
   indicator_prod <- which( market == original_market_id)
   indicator_mkt <- which( market == unique(original_market_id))
 
-  ##
-
   X_rand_mkt <- blp_data$data$X_rand[indicator_prod,,drop=FALSE]
   drawsRcMktShape_mkt <- blp_data$integration$drawsRcMktShape[indicator_mkt,,drop=FALSE]
   if(blp_data$parameters$total_demogr > 0){
@@ -98,24 +102,22 @@ dstdtheta_wrap <- function(  blp_data,
   }else{
     drawsDemMktShape_mkt<- matrix(NA)
   }
-  sij_mkt <- cm_input$sij[indicator_prod,,drop=FALSE]
-
-  cat("Evaluated at par_theta2:\n")
-  print(c(cm_input$theta2$par_theta2))
+  sij_mkt <- sij[indicator_prod,,drop=FALSE]
 
+  ## calculate dstdtheta_c in the given market
   out <- dstdtheta_c( sijt_arma = sij_mkt,
-                      indices = cm_input$theta2$indices,
-                      xt_arma = X_rand_mkt,
+                      indices = start_theta2$indices,
+                      xt_arma = blp_data$data$X_rand[indicator_prod,,drop=FALSE],
                       qvt_arma = drawsRcMktShape_mkt,
                       dt_arma = drawsDemMktShape_mkt,
                       weights_arma = blp_data$integration$weights )
+
+  ## preparing output
   rownames(out) <- paste0("share_" ,original_product_id[indicator_prod])
-
-
-  names_par <- kronecker( cm_input$theta2$final_col_names_par ,
-                          cm_input$theta2$final_row_names_par, paste, sep="*")
-  relevantRcDem_index <- cm_input$theta2$indices[,"row"] +
-    max( cm_input$theta2$indices[,"row"] ) * ( cm_input$theta2$indices[,"col"] - 1 )
+  names_par <- kronecker( start_theta2$final_col_names_par ,
+                          start_theta2$final_row_names_par, paste, sep="*")
+  relevantRcDem_index <- start_theta2$indices[,"row"] +
+    max( start_theta2$indices[,"row"] ) * ( start_theta2$indices[,"col"] - 1 )
   colnames(out) <- names_par[relevantRcDem_index]
 
   return(out)
@@ -127,15 +129,21 @@ dstdtheta_wrap <- function(  blp_data,
 #' Calculates derivatives of all shares with respect to all mean utilities in a given market.
 #'
 #' @param blp_data data object created by the function \code{BLP_data},
-#' @param cm_input contraction mapping object created by the function \code{getDelta_wrap},
-#' @param market character specifying the market in which derivatives are calculated
+#' @param par_theta2 matrix with column and rownames providing a starting value for the optimization routine (see details),
+#' @param market character specifying the market in which derivatives are calculated,
+#' @param printLevel level of output information (default = 1)
 #'
 #' @return Returns a numeric matrix with derivatives.
 #' Cell in row i and col j is the derivative of share i with respect to mean utility j.
 #'
+#' @details NA's in \code{par_theta2} entries indicate the exclusion from estimation, i.e. the coefficient is assumed to be zero.
+#' If only unobserved heterogeneity is used (no demographics), the column name of \code{par_theta2} must be "unobs_sd".
+#' With demographics the colnames must match the names of provided demographics (as in \code{demographic_draws}) and "unobs_sd".
+#' Row names of \code{par_theta2} must match random coefficients as specified in \code{model}. Constants must be named "(Intercept)".
+#'
 #' @examples
 #' K<-2 #number of random coefficients
-#' data <- get_BLP_dataset(nmkt = 25, nbrn = 20,
+#' data <- simulate_BLP_dataset(nmkt = 25, nbrn = 20,
 #'                         Xlin = c("price", "x1", "x2", "x3", "x4", "x5"),
 #'                         Xexo = c("x1", "x2", "x3", "x4", "x5"),
 #'                         Xrandom = paste0("x",1:K),instruments = paste0("iv",1:10),
@@ -162,62 +170,65 @@ dstdtheta_wrap <- function(  blp_data,
 #'                      integration_accuracy = 40,
 #'                      integration_seed = 1)
 #'
-#' theta_guesses <- matrix(c(0.5,2), nrow=2)
-#' rownames(theta_guesses) <- c("x1","x2")
-#' colnames(theta_guesses) <- "unobs_sd"
-#'
-#' blp_est <- estimateBLP(blp_data =blp_data,
-#'                        par_theta2 = theta_guesses,
-#'                        extremumCheck = FALSE ,
-#'                        printLevel = 1 )
-#' summary(blp_est)
-#'
-#' delta_eval <- getDelta_wrap(  blp_data=blp_data,
-#'                               par_theta2 = theta_guesses,
-#'                               printLevel = 4)
+#' theta2 <- matrix(c(0.5,2), nrow=2)
+#' rownames(theta2) <- c("x1","x2")
+#' colnames(theta2) <- "unobs_sd"
 #'
 #' derivatives2 <- dstddelta_wrap(  blp_data=blp_data,
-#'                                  cm_input = delta_eval,
+#'                                   par_theta2 = theta2,
 #'                                  market = 2)
-#'
 #' @export
-dstddelta_wrap <- function(  blp_data,
-
-                             cm_input, # input from contraction mapping
-                             market ){
-
+dstddelta_wrap <- function(   blp_data, par_theta2, market, printLevel = 1 ){
   nobs <- blp_data$parameters$nobs
   K <- blp_data$parameters$K
-  ## class checks
-  if(class(blp_data) != "blp_data")
-    stop("blp_data has wrong class. Call BLP_data() first.")
-
-  if(class(cm_input) != "blp_cm")
-    stop("blp_cm has wrong class. Call getDelta_wrap() first.")
-
-  ## find numeric market_id
-  if( missing(market))
-    stop("Specify a valid market.")
-
-  if( (length(market) != 1)  )
-    stop("Only one market can be specified.")
-
   original_market_id <- blp_data$parameters$market_id_char_in
   original_product_id <- blp_data$parameters$product_id
   market <- as.character(market)
 
-  if( !any(market %in% original_market_id) )
-    stop("Market is not available in provided dataset.")
+  ## check inouts
+  if(class(blp_data) != "blp_data") stop("Input has wrong class. Call BLP_data() first.")
+  if( printLevel > 0) cat("Mean utility (delta) is used as provided in the BLP_data() function.")
+  if( missing(market)) stop("Specify a valid market.")
+  if( (length(market) != 1)  ) stop("Only one market can be specified.")
+  if( !any(market %in% original_market_id) ) stop("Market is not available in provided dataset.")
 
+  ## calculate share evaluations (all markets)
+  current_delta <- blp_data$data$delta
+  start_theta2 <- .prepare_theta2(par_theta2,
+                                  final_col_names_par = c( "unobs_sd" ,
+                                                           blp_data$parameters$demographic_names),
+                                  final_row_names_par = colnames(blp_data$data$X_rand),
+                                  K = blp_data$parameters$K,
+                                  M = blp_data$parameters$total_demogr)
+  theta2Mat<- .get.theta2.reshape(theta2.in       = start_theta2$par_theta2,
+                                  totalRC         = blp_data$parameters$K,
+                                  total.demogr.in = blp_data$parameters$total_demogr,
+                                  indices.in      = start_theta2$indices,
+                                  fill            = 0 ) # NA are replaced by zeros to simplify x * par in getExpMu
+  expmu <-  getExpMu( theta2Matrix = theta2Mat,
+                      qv = blp_data$integration$drawsRcMktShape,
+                      Xrandom = blp_data$data$X_rand,
+                      cdid = blp_data$parameters$market_id,
+                      demographics = blp_data$integration$drawsDemMktShape)
+  sij <-  getSij(expmu = expmu,
+                 expdelta = exp(current_delta),
+                 cdindex = blp_data$parameters$cdindex )
+
+  ## extract market information
   indicator_prod <- which( market == original_market_id)
   indicator_mkt <- which( market == unique(original_market_id))
 
-  ##
-  sij_mkt <- cm_input$sij[indicator_prod,,drop=FALSE]
+  X_rand_mkt <- blp_data$data$X_rand[indicator_prod,,drop=FALSE]
+  drawsRcMktShape_mkt <- blp_data$integration$drawsRcMktShape[indicator_mkt,,drop=FALSE]
+  if(blp_data$parameters$total_demogr > 0){
+    drawsDemMktShape_mkt <- blp_data$integration$drawsDemMktShape[indicator_mkt,,drop=FALSE]
+  }else{
+    drawsDemMktShape_mkt<- matrix(NA)
+  }
+  sij_mkt <- sij[indicator_prod,,drop=FALSE]
 
-  cat("Evaluated at par_theta2:\n")
-  print(c(cm_input$theta2$par_theta2))
 
+  ## calculate dstdtheta_c in the given market
   out <- dstddelta_c( sijt = sij_mkt,
                       weights= blp_data$integration$weights )
   colnames(out) <- paste0("meanUtility_" ,original_product_id[indicator_prod])
@@ -246,7 +257,7 @@ dstddelta_wrap <- function(  blp_data,
 #'
 #' @examples
 #' K<-2 #number of random coefficients
-#' data <- get_BLP_dataset(nmkt = 25, nbrn = 20,
+#' data <- simulate_BLP_dataset(nmkt = 25, nbrn = 20,
 #'                         Xlin = c("price", "x1", "x2", "x3", "x4", "x5"),
 #'                         Xexo = c("x1", "x2", "x3", "x4", "x5"),
 #'                         Xrandom = paste0("x",1:K),instruments = paste0("iv",1:10),
@@ -317,22 +328,23 @@ gmm_obj_wrap <- function( blp_data, par_theta2, printLevel = 2){
                                   M = blp_data$parameters$total_demogr)
 
 
+
+
   # global variables with blp_results:
   blp_results <- new.env( parent = emptyenv())
   blp_results$deltaOld <- blp_data$data$delta
   blp_results$innerItAll <- c()
   blp_results$negShares<- FALSE
   blp_results$gradient <- rep(NA_real_, start_theta2$total_par )
-
-
   innerItAll_out <- blp_results$innerItAll
-  blp_results$deltaOld <- rep(0,nobs) #reset environment for final evaluation
 
+  start <- Sys.time()
   finalTmp <- gmm_obj(par_theta2 = start_theta2$par_theta2,####
                       indices=start_theta2$indices,
                       blp_results=blp_results,
                       blp_data=blp_data,
-                      printLevel=3)
+                      printLevel=printLevel)
+  end <- Sys.time()
 
   delta_out<- blp_results$deltaOld
   theta_rc_out <- start_theta2$par_theta2
@@ -343,7 +355,7 @@ gmm_obj_wrap <- function( blp_data, par_theta2, printLevel = 2){
   jacob_out <- blp_results$jacobian
   xi_out <- blp_results$xi
 
-  print(local_min_out)
+  if( printLevel > 0) print(local_min_out)
   out <- list("delta" = delta_out,
               "theta_rc" = theta_rc_out ,
               "theta_lin" = theta_lin_out ,
@@ -351,7 +363,8 @@ gmm_obj_wrap <- function( blp_data, par_theta2, printLevel = 2){
               "local_min" = local_min_out ,
               "gradient" = gradient_out ,
               "jacob" = jacob_out,
-              "xi" = xi_out )
+              "xi" = xi_out,
+              "time" = end - start)
   names(out$delta) <- paste0( blp_data$parameters$product_id , "_" ,
                               blp_data$parameters$market_id_char_in )
 
@@ -374,7 +387,7 @@ gmm_obj_wrap <- function( blp_data, par_theta2, printLevel = 2){
 #'
 #' @param blp_data data object created by the function \code{BLP_data},
 #' @param par_theta2 matrix with column and rownames providing a starting value for the optimization routine (see details),
-#' @param printLevel level of output information ranges from 1 (no GMM results) to 4 (every norm in the contraction mapping)
+#' @param printLevel level of output information (default = 1)
 #'
 #' @return Returns an object of class "blp_cm" with results from the contraction mapping.
 #' \describe{
@@ -391,7 +404,7 @@ gmm_obj_wrap <- function( blp_data, par_theta2, printLevel = 2){
 #'
 #' @examples
 #' K<-2 #number of random coefficients
-#' data <- get_BLP_dataset(nmkt = 25, nbrn = 20,
+#' data <- simulate_BLP_dataset(nmkt = 25, nbrn = 20,
 #'                         Xlin = c("price", "x1", "x2", "x3", "x4", "x5"),
 #'                         Xexo = c("x1", "x2", "x3", "x4", "x5"),
 #'                         Xrandom = paste0("x",1:K),instruments = paste0("iv",1:10),
@@ -427,7 +440,7 @@ gmm_obj_wrap <- function( blp_data, par_theta2, printLevel = 2){
 #'                               printLevel = 4)
 #'
 #' @export
-getDelta_wrap <- function(blp_data, par_theta2, printLevel = 4){
+getDelta_wrap <- function(blp_data, par_theta2, printLevel = 1){
   nobs <- blp_data$parameters$nobs
   K <- blp_data$parameters$K
   ## BLP_data class
@@ -486,17 +499,17 @@ getDelta_wrap <- function(blp_data, par_theta2, printLevel = 4){
 }
 
 
-#' Calculates predicted shares for a given set of non-linear parameters and data.
+#' Calculates information related to predicted shares for a given set of non-linear parameters and data.
 #'
-#' @param blp_data data object created by the function \code{BLP_data},
+#' @param blp_data data object created by the function \code{BLP_data} (provides, among others, mean utilitys and integration draws),
 #' @param par_theta2 matrix with column and rownames providing the evaluation point (see details),
-#' @param printLevel level of output information ranges from 1 (no GMM results) to 4 (every norm in the contraction mapping)
+#' @param printLevel level of output information (default = 1)
 #'
-#' @return Returns a numeric vector with shares.
+#' @return Returns a list with information related to predicted shares.
 #'
 #' @examples
 #' K<-2 #number of random coefficients
-#' data <- get_BLP_dataset(nmkt = 25, nbrn = 20,
+#' data <- simulate_BLP_dataset(nmkt = 25, nbrn = 20,
 #'                         Xlin = c("price", "x1", "x2", "x3", "x4", "x5"),
 #'                         Xexo = c("x1", "x2", "x3", "x4", "x5"),
 #'                         Xrandom = paste0("x",1:K),instruments = paste0("iv",1:10),
@@ -526,18 +539,26 @@ getDelta_wrap <- function(blp_data, par_theta2, printLevel = 4){
 #' rownames(theta_guesses) <- c("x1","x2")
 #' colnames(theta_guesses) <- "unobs_sd"
 #'
-#' shares <- getShares_wrap(  blp_data=blp_data,
+#' shares <- getShareInfo(  blp_data=blp_data,
 #'                            par_theta2 = theta_guesses,
 #'                            printLevel = 4)
 #'
 #' @export
-getShares_wrap <- function(blp_data, par_theta2, printLevel = 4){
+getShareInfo <- function(blp_data, par_theta2, printLevel = 1){
   nobs <- blp_data$parameters$nobs
   K <- blp_data$parameters$K
   ## BLP_data class
   if(class(blp_data) != "blp_data")
     stop("Input has wrong class. Call BLP_data() first.")
 
+  ## mean utility
+  if( printLevel > 0){
+    cat("Mean utility (delta) is used as provided in the BLP_data() function.")
+  }
+
+  current_delta <- blp_data$data$delta
+
+
   ## check and prepare par_theta2
 
   start_theta2 <- .prepare_theta2(par_theta2,
@@ -553,25 +574,48 @@ getShares_wrap <- function(blp_data, par_theta2, printLevel = 4){
                                   indices.in      = start_theta2$indices,
                                   fill            = 0 ) # NA are replaced by zeros to simplify x * par in getExpMu
 
-  #Call C++ function:
+  colnames(theta2Mat) <- c( "unobs_sd" ,
+                            blp_data$parameters$demographic_names)
+  rownames(theta2Mat) <- colnames(blp_data$data$X_rand)
+
+
+  # get the exp of individual part of utility:
   expMu <- getExpMu( theta2Matrix = theta2Mat,
                      qv = blp_data$integration$drawsRcMktShape,
                      Xrandom = blp_data$data$X_rand,
                      cdid = blp_data$parameters$market_id,
                      demographics = blp_data$integration$drawsDemMktShape ) ;
 
+  rownames(expMu) <- paste0("expMu_",
+                          blp_data$parameters$product_id , "_" ,
+                          blp_data$parameters$market_id_char_in )
+  colnames(expMu) <- paste0("individual_", 1: blp_data$integration$amountDraws)
 
+  # calculate individual choice probabilities
   Sij <- getSij(expmu = expMu,
-                expdelta = exp(blp_data$data$delta),
+                expdelta = exp(current_delta),
                 cdindex = blp_data$parameters$cdindex )
+  rownames(Sij) <- paste0("share_",
+                              blp_data$parameters$product_id , "_" ,
+                              blp_data$parameters$market_id_char_in )
+  colnames(Sij) <- paste0("individual_", 1: blp_data$integration$amountDraws)
 
+  # calc. aggregated choice probabilities, i.e. shares
   shares <- c( Sij %*% blp_data$integration$weights)
 
 
   names(shares) <- paste0( blp_data$parameters$product_id , "_" ,
                            blp_data$parameters$market_id_char_in )
 
-  return(shares)
+  out <- list( "shares"=shares,
+               "sij" = Sij,
+               "expMu" =  expMu,
+               "theta2" = theta2Mat)
+
+  class(out) <- "shareInfo"
+
+  return(out)
+
 
 
 }
@@ -582,7 +626,7 @@ getShares_wrap <- function(blp_data, par_theta2, printLevel = 4){
 #'
 #' @param blp_data data object created by the function \code{BLP_data},
 #' @param par_theta2 matrix with column and rownames providing the evaluation point (see details),
-#' @param printLevel level of output information ranges from 1 (no GMM results) to 4 (every norm in the contraction mapping)
+#' @param printLevel level of output information (default = 1)
 #'
 #' @return Returns a matrix with the jacobian (products in rows, parameters in columns).
 #'
@@ -593,7 +637,7 @@ getShares_wrap <- function(blp_data, par_theta2, printLevel = 4){
 #'
 #' @examples
 #' K<-2 #number of random coefficients
-#' data <- get_BLP_dataset(nmkt = 25, nbrn = 20,
+#' data <- simulate_BLP_dataset(nmkt = 25, nbrn = 20,
 #'                         Xlin = c("price", "x1", "x2", "x3", "x4", "x5"),
 #'                         Xexo = c("x1", "x2", "x3", "x4", "x5"),
 #'                         Xrandom = paste0("x",1:K),instruments = paste0("iv",1:10),
@@ -629,7 +673,7 @@ getShares_wrap <- function(blp_data, par_theta2, printLevel = 4){
 #'                              printLevel = 2)
 #' head(jacobian)
 #' @export
-getJacobian_wrap <- function( blp_data, par_theta2, printLevel = 2){
+getJacobian_wrap <- function( blp_data, par_theta2, printLevel = 1){
 
   nobs <- blp_data$parameters$nobs
   K <- blp_data$parameters$K
@@ -637,6 +681,12 @@ getJacobian_wrap <- function( blp_data, par_theta2, printLevel = 2){
   if(class(blp_data) != "blp_data")
     stop("Input has wrong class. Call BLP_data() first.")
 
+  ## mean utility
+  if( printLevel > 0){
+    cat("Mean utility (delta) is used as provided in the BLP_data() function.")
+  }
+
+  current_delta <- blp_data$data$delta
 
   ## check and prepare par_theta2
 
@@ -661,7 +711,7 @@ getJacobian_wrap <- function( blp_data, par_theta2, printLevel = 2){
                       demographics = blp_data$integration$drawsDemMktShape)
 
   sij <-  getSij(expmu = expmu,
-                 expdelta = c(exp(blp_data$data$delta)),
+                 expdelta = exp(current_delta),
                  cdindex = blp_data$parameters$cdindex )
 
   jacobian <- jacob_c(sij = sij,
